/* config.h.  Generated from config.h.in by configure.  */
/* config.h.in.  Generated from configure.ac by autoheader.  */

/* "have alsa" */
#define HAVE_ALSA 1

/* Define to 1 if you have the <alsa/asoundlib.h> header file. */
#define HAVE_ALSA_ASOUNDLIB_H 1

/* Define to 1 if you have the <dev/bktr/ioctl_bt848.h> header file. */
/* #undef HAVE_DEV_BKTR_IOCTL_BT848_H */

/* Define to 1 if you have the <dev/ic/bt8xx.h> header file. */
/* #undef HAVE_DEV_IC_BT8XX_H */

/* Define to 1 if you have the `dlopen' function. */
/* #undef HAVE_DLOPEN */

/* "have dvb" */
#define HAVE_DVB 1

/* Define to 1 if you have the `fseeko' function. */
#define HAVE_FSEEKO 1

/* Define to 1 if you have the `ftello' function. */
#define HAVE_FTELLO 1

/* Define to 1 if you have the `getnameinfo' function. */
#define HAVE_GETNAMEINFO 1

/* Define to 1 if you have the <getopt.h> header file. */
#define HAVE_GETOPT_H 1

/* Define to 1 if you have the `getopt_long' function. */
#define HAVE_GETOPT_LONG 1

/* Define to 1 if you have the `getpt' function. */
#define HAVE_GETPT 1

/* "have opengl" */
#define HAVE_GL 1

/* Define to 1 if you have the `iconv_open' function. */
#define HAVE_ICONV_OPEN 1

/* Define to 1 if you have the <inttypes.h> header file. */
#define HAVE_INTTYPES_H 1

/* "have lirc" */
#define HAVE_LIBLIRC_CLIENT 1

/* "have mad" */
#define HAVE_LIBMAD 1

/* "have libmpeg2" */
#define HAVE_LIBMPEG2 1

/* Define to 1 if you have the `Xdpms' library (-lXdpms). */
#define HAVE_LIBXDPMS 1

/* Define to 1 if you have the `Xinerama' library (-lXinerama). */
#define HAVE_LIBXINERAMA 1

/* Define to 1 if you have the `Xrender' library (-lXrender). */
#define HAVE_LIBXRENDER 1

/* Define to 1 if you have the `Xv' library (-lXv). */
#define HAVE_LIBXV 1

/* Define to 1 if you have the <linux/joystick.h> header file. */
#define HAVE_LINUX_JOYSTICK_H 1

/* Define to 1 if you have the `memmem' function. */
#define HAVE_MEMMEM 1

/* Define to 1 if you have the <memory.h> header file. */
#define HAVE_MEMORY_H 1

/* "have ipv6" */
#define HAVE_SOCKADDR_STORAGE 1

/* Define to 1 if you have the <soundcard.h> header file. */
/* #undef HAVE_SOUNDCARD_H */

/* Define to 1 if you have the <stdint.h> header file. */
#define HAVE_STDINT_H 1

/* Define to 1 if you have the <stdlib.h> header file. */
#define HAVE_STDLIB_H 1

/* Define to 1 if you have the `strcasestr' function. */
#define HAVE_STRCASESTR 1

/* Define to 1 if you have the <strings.h> header file. */
#define HAVE_STRINGS_H 1

/* Define to 1 if you have the <string.h> header file. */
#define HAVE_STRING_H 1

/* Define to 1 if you have the <sys/select.h> header file. */
#define HAVE_SYS_SELECT_H 1

/* Define to 1 if you have the <sys/soundcard.h> header file. */
#define HAVE_SYS_SOUNDCARD_H 1

/* Define to 1 if you have the <sys/stat.h> header file. */
#define HAVE_SYS_STAT_H 1

/* Define to 1 if you have the <sys/types.h> header file. */
#define HAVE_SYS_TYPES_H 1

/* Define to 1 if you have the <unistd.h> header file. */
#define HAVE_UNISTD_H 1

/* "dvb2 supportin libzvbi" */
#define HAVE_VBI_DVB2 1

/* "proxy support in libzvbi" */
#define HAVE_VBI_PROXY 1

/* "have xft" */
#define HAVE_XFT 1

/* "have xmu" */
#define HAVE_XMU 1

/* "have zvbi" */
#define HAVE_ZVBI 1

/* Define to the address where bug reports for this package should be sent. */
#define PACKAGE_BUGREPORT ""

/* Define to the full name of this package. */
#define PACKAGE_NAME ""

/* Define to the full name and version of this package. */
#define PACKAGE_STRING ""

/* Define to the one symbol short name of this package. */
#define PACKAGE_TARNAME ""

/* Define to the version of this package. */
#define PACKAGE_VERSION ""

/* Define to 1 if you have the ANSI C header files. */
#define STDC_HEADERS 1

/* Define to 1 if the X Window System is missing or not being used. */
/* #undef X_DISPLAY_MISSING */
