#ifndef DVB_STREAM
#define DVB_STREAM


#include "dvb_struct.h"

/* ----------------------------------------------------------------------- */
int write_stream(struct dvb_state *h, char *filename, int sec) ;

#endif
